"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var errors_1 = require("../errors");
var spying_1 = require("../spying");
/**
 * Gives functionality to ensure the outcome of a test is as expected
 */
var Matcher = (function () {
    function Matcher(actualValue) {
        this._shouldMatch = true;
        this._actualValue = actualValue;
    }
    Object.defineProperty(Matcher.prototype, "actualValue", {
        get: function () {
            return this._actualValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Matcher.prototype, "shouldMatch", {
        get: function () {
            return this._shouldMatch;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Matcher.prototype, "not", {
        /**
         * Any subsequent matcher function will be looking for the opposite criteria
         */
        get: function () {
            this._shouldMatch = !this.shouldMatch;
            return this;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Checks that a value is identical to another
     * @param expectedValue - the value that will be used to match
     */
    Matcher.prototype.toBe = function (expectedValue) {
        if ((expectedValue !== this._actualValue) === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, expectedValue, this.shouldMatch);
        }
    };
    /**
     * Checks that a value is equal to another (for objects the function will check for deep equality)
     * @param expectedValue - the value that will be used to match
     */
    Matcher.prototype.toEqual = function (expectedValue) {
        var valueMatch = expectedValue instanceof spying_1.TypeMatcher
            ? expectedValue.test(this._actualValue)
            :
                expectedValue == this._actualValue; // tslint:disable-line:triple-equals
        if (valueMatch !== this.shouldMatch) {
            if (typeof expectedValue !== "object" ||
                this._checkObjectsAreDeepEqual(expectedValue, this._actualValue) !==
                    this.shouldMatch) {
                throw new errors_1.EqualMatchError(this._actualValue, expectedValue, this.shouldMatch);
            }
        }
    };
    /**
     * Checks that a value is not undefined
     */
    Matcher.prototype.toBeDefined = function () {
        if ((this._actualValue === undefined) === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, undefined, !this.shouldMatch);
        }
    };
    /**
     * Checks that a value is null
     */
    Matcher.prototype.toBeNull = function () {
        if ((this._actualValue !== null) === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, null, this.shouldMatch);
        }
    };
    /**
     * Checks that a value is equivalent to boolean true
     */
    Matcher.prototype.toBeTruthy = function () {
        if ((this._actualValue && !this.shouldMatch) ||
            (!this._actualValue && this.shouldMatch)) {
            throw new errors_1.TruthyMatchError(this._actualValue, this.shouldMatch);
        }
    };
    Matcher.prototype._checkObjectsAreDeepEqual = function (objectA, objectB) {
        // if one object is an array and the other is not then they are not equal
        if (Array.isArray(objectA) !== Array.isArray(objectB)) {
            return false;
        }
        // get all the property keys for each object
        var OBJECT_A_KEYS = Object.keys(objectA);
        var OBJECT_B_KEYS = Object.keys(objectB);
        // if they don't have the same amount of properties then clearly not
        if (OBJECT_A_KEYS.length !== OBJECT_B_KEYS.length) {
            return false;
        }
        // check all the properties of each object
        for (var _i = 0, OBJECT_A_KEYS_1 = OBJECT_A_KEYS; _i < OBJECT_A_KEYS_1.length; _i++) {
            var objectAKey = OBJECT_A_KEYS_1[_i];
            // if the property values are not the same
            if (objectA[objectAKey] !== objectB[objectAKey]) {
                // and it's not an object or the objects are not equal
                if (typeof objectA[objectAKey] !== "object" ||
                    this._checkObjectsAreDeepEqual(objectA[objectAKey], objectB[objectAKey]) === false) {
                    // then not deep equal
                    return false;
                }
            }
        }
        // all properties match so all is good
        return true;
    };
    return Matcher;
}());
exports.Matcher = Matcher;
//# sourceMappingURL=matcher.js.map